from django.conf import settings
import requests
from mynewsdesk import models


def request(service, params={}):
    url = 'http://www.mynewsdesk.com/services/pressroom/' + service + '/' + settings.MYNEWSDESK_KEY + '/'
    return requests.get(url, params=params)


def json_request(service, params={}):
    params['format'] = 'json'
    r = request(service, params)
    return r.json()


def get_list(type_of_media=models.TYPE_PRESSRELEASE, limit=20, offset=0, order=False, callback=False, pressroom=False, archived=False):
    params = {
        'type_of_media': type_of_media,
        'limit': limit,
        'offset': offset
    }

    if pressroom:
        params['pressroom'] = pressroom

    if order:
        params['order'] = order

    if archived:
        params['archived'] = 'true'

    if callback:
        params['callback'] = callback

    return json_request('list', params)

def subscribe(email, types_list):
    params = {
        'newsdesk_subscriber_email': email
    }
    for stype in types_list:
        params['newsdesk_subscribe_to_' + models.SUBSCRIBE_ENDINGS[stype]] = 1
    r = request('subscribe', params)
    if r.status_code == requests.codes.ok:
        return True
    else:
        return False

# TODO: method for 'view' service
# TODO: method for 'search' service
# TODO: method for 'pressroom_info' service
# TODO: method for 'create_comment' service


from mynewsdesk import models, api

def add_report(report, result):
    report['created'] += result['created']
    report['updated'] += result['updated']
    report['errors'] += result['errors']
    return report

def sync_all():
    report = {
        'created': 0,
        'updated': 0,
        'errors': 0
    }

    r = sync_list(models.TYPE_PRESSRELEASE)
    report = add_report(report, r)

    r = sync_list(models.TYPE_NEWS)
    report = add_report(report, r)

    r = sync_list(models.TYPE_BLOG_POST)
    report = add_report(report, r)

    r = sync_list(models.TYPE_CONTACT_PERSON)
    report = add_report(report, r)

    r = sync_list(models.TYPE_DOCUMENT)
    report = add_report(report, r)

    r = sync_list(models.TYPE_EVENT)
    report = add_report(report, r)

    r = sync_list(models.TYPE_IMAGE)
    report = add_report(report, r)

    r = sync_list(models.TYPE_VIDEO)
    report = add_report(report, r)

    return report

def add_link(material, link):
    db_link = models.Link(material=material, url=link['url'], text=link['text'])
    db_link.save()

def sync_list(service=models.TYPE_PRESSRELEASE):
    item_list = api.get_list(service)
    report = {
        'created': 0,
        'updated': 0,
        'errors': 0
    }
    if item_list and 'items' in item_list.keys() and 'item' in item_list['items']:
        for item in item_list['items']['item']:
            try:
                try:
                    db_item = models.Material.objects.get(id=item['id'])
                    report['updated'] += 1
                except models.Material.DoesNotExist:
                    db_item = models.Material(id=item['id'])
                    report['created'] += 1

                for key, val in item.items():
                    if not isinstance(val, list) and not (type(val) is dict):
                        setattr(db_item, key, val)

                db_item.save()

                for key, val in item.items():
                    if isinstance(val, list) or (type(val) is dict):
                        if 'channels' == key:
                            for channel in val['channel']:
                                try:
                                    db_channel = models.Channel.objects.get(id=channel['id'])
                                except:
                                    db_channel = models.Channel(id=channel['id'], title=channel['text'])
                                    db_channel.save()
                                db_item.channels.add(db_channel)
                        elif 'event_types' == key:
                            for event_type in val['event_type']:
                                try:
                                    db_event_type = models.EventType.objects.get(id=event_type['id'])
                                except:
                                    db_event_type = models.EventType(id=event_type['id'], title=event_type['text'])
                                    db_event_type.save()
                                db_item.event_types.add(db_event_type)
                        elif 'tags' == key:
                            for tag in val['tag']:
                                try:
                                    db_tag = models.Tag.objects.get(name=tag)
                                except:
                                    db_tag = models.Tag(name=tag)
                                    db_tag.save()
                                db_item.tags.add(db_tag)

                        elif 'links' == key:
                            models.Link.objects.filter(material=db_item).delete()
                            if isinstance(val['link'], list):
                                for link in val['link']:
                                    add_link(db_item, link)
                            else:
                                add_link(db_item, val['link'])

                        # TODO: subjects
                        # TODO: geographic_areas
                        # TODO: contact_peoples
                        # TODO: related_items
                        # TODO: instant_messaging
                        # TODO: comments
            except:
                report['errors'] += 1

    return report



from django.db import models

# Material types

TYPE_PRESSRELEASE = 'pressrelease'
TYPE_NEWS = 'news'
TYPE_BLOG_POST = 'blog_post'
TYPE_EVENT = 'event'
TYPE_IMAGE = 'image'
TYPE_VIDEO = 'video'
TYPE_DOCUMENT = 'document'
TYPE_CONTACT_PERSON = 'contact_person'

SUBSCRIBE_ENDINGS = {
    TYPE_PRESSRELEASE: 'pressreleases',
    TYPE_NEWS: 'news',
    TYPE_BLOG_POST: 'blog_posts',
    TYPE_EVENT: 'events',
    TYPE_IMAGE: 'images',
    TYPE_VIDEO: 'videos',
    TYPE_DOCUMENT: 'documents',
    TYPE_CONTACT_PERSON: 'contact_person'
}

class Tag(models.Model):
    url = models.URLField(blank=True, null=True)
    name = models.CharField(max_length=255, primary_key=True)
    level = models.IntegerField(blank=True, null=True)

# class Pressroom(models.Model):
#     id = models.IntegerField(primary_key=True)
#     country = models.CharField(max_length=255, blank=True, null=True)
#     country_code = models.CharField(max_length=3, blank=True, null=True)
#     source_name = models.CharField(max_length=255, blank=True, null=True)
#     source_id = models.CharField(max_length=255, blank=True, null=True)
#     header = models.TextField(blank=True, null=True)
#     description = models.TextField(blank=True, null=True)
#     address_street = models.TextField(blank=True, null=True)
#     address_postal_code = models.TextField(blank=True, null=True)
#     address_city = models.TextField(blank=True, null=True)
#     address_country = models.TextField(blank=True, null=True)
#
#     tag_cloud = models.ManyToManyField(Tag)

# class Logotype(models.Model):
#     pressroom = models.ForeignKey(Pressroom, related_name='logotypes')
#     size = models.CharField(max_length=50, blank=True, null=True)
#     image = models.URLField()

class Channel(models.Model):
    id = models.IntegerField(primary_key=True)
    title = models.CharField(max_length=255)

# class Subject(models.Model):
#     id = models.CharField(max_length=255, primary_key=True)
#     parent_id = models.CharField(max_length=255, null=True)
#     name = models.TextField()

# class GeographicArea(models.Model):
#     id = models.CharField(max_length=255, primary_key=True)
#     parent_id = models.CharField(max_length=255, null=True)
#     name = models.TextField()

class EventType(models.Model):
    id = models.CharField(max_length=255, primary_key=True)
    name = models.TextField()

class Material(models.Model):
    id = models.IntegerField(primary_key=True)
    type_of_media = models.CharField(max_length=15)
    language = models.CharField(max_length=10)

    channels = models.ManyToManyField(Channel, related_name='materials', blank=True)

    source_id = models.IntegerField(blank=True, null=True)
    source_name = models.CharField(max_length=255, blank=True, null=True)
    pressroom_name = models.CharField(max_length=255, blank=True, null=True)
    pressroom_id = models.IntegerField(blank=True, null=True)
    pressroom = models.CharField(max_length=10, blank=True, null=True)
    organization_number = models.CharField(max_length=50, blank=True, null=True)
    url = models.URLField(blank=True, null=True)

    published_at = models.DateTimeField()
    created_at = models.DateTimeField()
    updated_at = models.DateTimeField()

    header = models.TextField(blank=True, null=True)
    summary = models.TextField(blank=True, null=True)
    body = models.TextField(blank=True, null=True)
    boilerplate = models.TextField(blank=True, null=True)

    # for contact person

    position = models.IntegerField(blank=True, null=True)
    name = models.CharField(max_length=255, blank=True, null=True)
    title = models.CharField(max_length=255, blank=True, null=True)
    specialist = models.CharField(max_length=255, blank=True, null=True)
    phone = models.CharField(max_length=255, blank=True, null=True)
    phone_alternative = models.CharField(max_length=255, blank=True, null=True)
    email = models.EmailField(blank=True, null=True)
    pressroom_contact = models.NullBooleanField(blank=True, null=True)

    # for event

    start_at = models.DateTimeField(blank=True, null=True)
    end_at = models.DateTimeField(blank=True, null=True)
    location = models.TextField(blank=True, null=True)
    signup_url = models.URLField(blank=True, null=True)
    event_types = models.ManyToManyField(EventType, related_name='materials', blank=True)

    # for image
    photographer = models.TextField(blank=True, null=True)
    image_format = models.CharField(max_length=10, blank=True, null=True)
    image_size = models.BigIntegerField(blank=True, null=True)
    image_dimensions = models.CharField(max_length=50, blank=True, null=True)
    download_url = models.URLField(blank=True, null=True)

    # for video
    flash_video = models.URLField(blank=True, null=True)
    flash_video_width = models.IntegerField(blank=True, null=True)
    flash_video_height = models.IntegerField(blank=True, null=True)
    embed_code = models.TextField(blank=True, null=True)
    thumbnail = models.URLField(blank=True, null=True)
    duration = models.CharField(max_length=50, blank=True, null=True)

    # for document
    document_format = models.CharField(max_length=10, blank=True, null=True)
    document_size = models.BigIntegerField(blank=True, null=True)
    document = models.URLField(blank=True, null=True)

    image = models.URLField(blank=True, null=True)
    image_thumbnail_large = models.URLField(blank=True, null=True)
    image_thumbnail_medium = models.URLField(blank=True, null=True)
    image_thumbnail_small = models.URLField(blank=True, null=True)

    attached_pdf = models.URLField(blank=True, null=True)
    attached_doc = models.URLField(blank=True, null=True)

    tags = models.ManyToManyField(Tag, related_name='materials', blank=True)
    # subjects = models.ManyToManyField(Subject, related_name='materials', blank=True, null=True)
    # geographic_areas = models.ManyToManyField(GeographicArea, related_name='materials', blank=True, null=True)

    # contact_people = models.ManyToManyField('self', blank=True, null=True)
    # related_items = models.ManyToManyField('self', blank=True, null=True)

# class InstantMessaging(models.Model):
#     material = models.ForeignKey(Material, related_name='instant_messaging')
#     type = models.CharField(max_length=255)
#     text = models.TextField()
#
# class Comment(models.Model):
#     material = models.ForeignKey(Material, related_name='comments')
#     author_url = models.URLField(blank=True, null=True)
#     author = models.CharField(max_length=255, blank=True, null=True)
#     text = models.TextField(blank=True, null=True)

class Link(models.Model):
    material = models.ForeignKey(Material, related_name='links')
    url = models.URLField(blank=True, null=True)
    text = models.TextField(blank=True, null=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mynewsdesk', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='material',
            old_name='type',
            new_name='type_of_media',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Channel',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('title', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('id', models.CharField(max_length=255, serialize=False, primary_key=True)),
                ('name', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(null=True, blank=True)),
                ('text', models.TextField(null=True, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Material',
            fields=[
                ('id', models.IntegerField(serialize=False, primary_key=True)),
                ('type', models.CharField(max_length=15)),
                ('language', models.CharField(max_length=10)),
                ('source_id', models.IntegerField(null=True, blank=True)),
                ('source_name', models.CharField(max_length=255, null=True, blank=True)),
                ('pressroom_name', models.CharField(max_length=255, null=True, blank=True)),
                ('pressroom_id', models.IntegerField(null=True, blank=True)),
                ('pressroom', models.CharField(max_length=10, null=True, blank=True)),
                ('organization_number', models.CharField(max_length=50, null=True, blank=True)),
                ('url', models.URLField(null=True, blank=True)),
                ('published_at', models.DateTimeField()),
                ('created_at', models.DateTimeField()),
                ('updated_at', models.DateTimeField()),
                ('header', models.TextField(null=True, blank=True)),
                ('summary', models.TextField(null=True, blank=True)),
                ('body', models.TextField(null=True, blank=True)),
                ('boilerplate', models.TextField(null=True, blank=True)),
                ('position', models.IntegerField(null=True, blank=True)),
                ('name', models.CharField(max_length=255, null=True, blank=True)),
                ('title', models.CharField(max_length=255, null=True, blank=True)),
                ('specialist', models.CharField(max_length=255, null=True, blank=True)),
                ('phone', models.CharField(max_length=255, null=True, blank=True)),
                ('phone_alternative', models.CharField(max_length=255, null=True, blank=True)),
                ('email', models.EmailField(max_length=254, null=True, blank=True)),
                ('pressroom_contact', models.NullBooleanField()),
                ('start_at', models.DateTimeField(null=True, blank=True)),
                ('end_at', models.DateTimeField(null=True, blank=True)),
                ('location', models.TextField(null=True, blank=True)),
                ('signup_url', models.URLField(null=True, blank=True)),
                ('photographer', models.TextField(null=True, blank=True)),
                ('image_format', models.CharField(max_length=10, null=True, blank=True)),
                ('image_size', models.BigIntegerField(null=True, blank=True)),
                ('image_dimensions', models.CharField(max_length=50, null=True, blank=True)),
                ('download_url', models.URLField(null=True, blank=True)),
                ('flash_video', models.URLField(null=True, blank=True)),
                ('flash_video_width', models.IntegerField(null=True, blank=True)),
                ('flash_video_height', models.IntegerField(null=True, blank=True)),
                ('embed_code', models.TextField(null=True, blank=True)),
                ('thumbnail', models.URLField(null=True, blank=True)),
                ('duration', models.CharField(max_length=50, null=True, blank=True)),
                ('document_format', models.CharField(max_length=10, null=True, blank=True)),
                ('document_size', models.BigIntegerField(null=True, blank=True)),
                ('document', models.URLField(null=True, blank=True)),
                ('image', models.URLField(null=True, blank=True)),
                ('image_thumbnail_large', models.URLField(null=True, blank=True)),
                ('image_thumbnail_medium', models.URLField(null=True, blank=True)),
                ('image_thumbnail_small', models.URLField(null=True, blank=True)),
                ('attached_pdf', models.URLField(null=True, blank=True)),
                ('attached_doc', models.URLField(null=True, blank=True)),
                ('channels', models.ManyToManyField(related_name='materials', null=True, to='mynewsdesk.Channel', blank=True)),
                ('event_types', models.ManyToManyField(related_name='materials', null=True, to='mynewsdesk.EventType', blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('url', models.URLField(null=True, blank=True)),
                ('name', models.CharField(max_length=255, serialize=False, primary_key=True)),
                ('level', models.IntegerField(null=True, blank=True)),
            ],
        ),
        migrations.AddField(
            model_name='material',
            name='tags',
            field=models.ManyToManyField(related_name='materials', null=True, to='mynewsdesk.Tag', blank=True),
        ),
        migrations.AddField(
            model_name='link',
            name='material',
            field=models.ForeignKey(related_name='links', to='mynewsdesk.Material'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from mynewsdesk import sync

class Command(BaseCommand):
    def handle(self, *args, **options):
        r = sync.sync_all()
        print r


import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-mynewsdesk',
    version='0.5',
    packages=['mynewsdesk'],
    include_package_data=True,
    license='MIT License',
    description='Django package for MyNewsDesk API',
    long_description=README,
    url='https://github.com/pkozlov/django-mynewsdesk',
    author='Pavel Kozlov',
    author_email='mail@pkozlov.ru',
    install_requires=[
        'Django >= 1.4.3',
        'requests >= 1.0.3'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)